import Constants

class Character:
    def __init__(self):
        self.name = ''
        self.base_health = 0
        self.health = 0
        self.max_health = 0
        self.base_mana = 0
        self.mana = 0
        self.base_stamina = 0
        self.stamina = 0
        self.base_defense = 0
        self.defense = 0
        self.buffs = []
        self.classes = {}
        self.classes['Warrior'] = 0
        self.classes['Wizard'] = 0
        self.recent_actions = []
        self.recent_trends = {}
        self.recent_trends['Warrior'] = [0]
        self.recent_trends['Wizard'] = [0]

    def tick(self):
        if self.health > 0:
            # restore pools
            if self.stamina < self.base_stamina:
                self.stamina = self.stamina + Constants.stamina_heal_per_tick
            if self.mana < self.base_mana:
                self.mana = self.mana + Constants.mana_heal_per_tick
            if self.health < self.max_health:
                self.health = self.health + Constants.health_heal_per_tick
                
            # walk and count-down the buffs
            for buff in self.buffs:
                buff.timer = buff.timer - 1
                if buff.timer == 0:
                    # remove the buff and effects
                    self.buffs.remove(buff)
                    if buff.effect_type == 'Health':
                        self.health = self.health - buff.effect_amount
                        if self.health < 1:
                            self.health = 1
                        self.max_health = self.max_health - buff.effect_amount
                    elif buff.effect_type == 'Mana':
                        self.mana = self.mana - buff.effect_amount
                        if self.mana < 0:
                            self.mana = 0
                    elif buff.effect_type == 'Stamina':
                        self.stamina = self.stamina - buff.effect_amount
                        if self.stamina < 0:
                            self.stamina = 0
                    elif buff.effect_type == 'Defense':
                        self.defense = self.defense - buff.effect_amount
                        if self.defense < 0:
                            self.defense = 0
                    # TODO send an expiration message

    def adjust_trend(self, action):
        self.recent_actions.append(action)
        if len(self.recent_actions) > Constants.review_after_recent_actions:
            # sum each action taken
            warrior_influence = 0
            wizard_influence = 0
            
            for skill in self.recent_actions:
                if skill.positive_class == 'Warrior':
                    warrior_influence = warrior_influence + skill.positive_value
                elif skill.positive_class == 'Wizard':
                    wizard_influence = wizard_influence + skill.positive_value
                
                if skill.negative_class == 'Warrior':
                    warrior_influence = warrior_influence - skill.negative_value
                elif skill.negative_class == 'Wizard':
                    wizard_influence = wizard_influence - skill.negative_value
            
            # append the sums to the trend influences list
            self.recent_trends['Warrior'].append(warrior_influence)
            self.recent_trends['Wizard'].append(wizard_influence)
            
            # recompute the character's current trend points in each line
            # Average the sum the previous influence and the current influence.
            warrior_influence = 0
            for influence in self.recent_trends['Warrior']:
                warrior_influence = warrior_influence + influence
            warrior_influence = warrior_influence / len(self.recent_trends['Warrior'])
            
            wizard_influence = 0
            for influence in self.recent_trends['Wizard']:
                wizard_influence = wizard_influence + influence
            wizard_influence = wizard_influence  / len(self.recent_trends['Wizard'])

            # Set that average to the new current influence.
            self.recent_trends['Warrior'] = [warrior_influence]
            self.recent_trends['Wizard'] = [wizard_influence]
            
            # If recent trends list is too large, remove the first element
            if len(self.recent_trends['Warrior']) > Constants.track_recent_trends:
                self.recent_trends['Warrior'] = self.recent_trends['Warrior'][1:]
            if len(self.recent_trends['Wizard']) > Constants.track_recent_trends:
                self.recent_trends['Wizard'] = self.recent_trends['Wizard'][1:]
                
            # Add the current influence to the current trend points, and set this as the current trend points.
            self.classes['Warrior'] = self.classes['Warrior'] + warrior_influence
            self.classes['Wizard'] = self.classes['Wizard'] + wizard_influence
            if self.classes['Warrior'] < 0:
                self.classes['Warrior'] = 0
            if self.classes['Wizard'] < 0:
                self.classes['Wizard'] = 0
            
            print self.name + ' influences: ' + str(self.classes)

            # Clear the recent actions list
            self.recent_actions = []

            # Adjust the base stamina and mana
            self.base_stamina = 30 + self.classes['Warrior']
            self.base_mana = 30 + self.classes['Wizard']
        
    def damage(self, amount):
        if self.health > 0:
            self.health = self.health - amount
            if self.health < 1:
                self.die()
    
    def tire(self, amount):
        if self.stamina > 0:
            self.stamina = self.stamina - amount
            if self.stamina < 0:
                self.stamina = 0
    
    def drain(self, amount):
        if self.mana > 0:
            self.mana = self.mana - amount
            if self.mana < 0:
                self.mana = 0
    
    def heal(self, amount):
        if self.health > 0 and self.health < self.max_health:
            self.health = self.health + amount
            if self.health > self.max_health:
                self.health = self.max_health
            print self.name + ' is healed for ' + str(amount) + '.'

    def buff(self, buff):
        self.buffs.append(buff)
        
        # apply the buff
        if buff.effect_type == 'Health':
            self.max_health = self.max_health + buff.effect_amount
            print self.name + ' is health buffed.'
        elif buff.effect_type == 'Defense':
            self.defense = self.defense + buff.effect_amount
            print self.name + ' is defense buffed.'

    def die(self):
        self.health = 0
        self.max_health = 0
        self.buffs = []
        print self.name + ' died.'
        
    def is_alive(self):
        return self.health > 0
    
    def is_dead(self):
        return self.health < 1

    def __str__(self):
        return self.name + ': health ' + str(self.health) + '/' + str(self.max_health) + '  stamina ' + str(self.stamina) + '/' + str(self.base_stamina) + '  mana ' + str(self.mana) + '/' + str(self.base_mana) + '  defense ' + str(self.defense) + '/' + str(self.base_defense) + '  levels ' + str(self.classes) + '  recent trends ' + str(self.recent_trends)