import Buff
import Character
import Constants
import Skill
import Skills

import random
import sys

class Sim:
    def __init__(self):
        self.player = Character.Character()
        self.player.name = 'Player'
        self.player.base_health = Constants.player_start_health
        self.player.health = Constants.player_start_health
        self.player.max_health = Constants.player_start_health
        self.player.base_stamina = Constants.player_start_stamina
        self.player.stamina = Constants.player_start_stamina
        self.player.base_mana = Constants.player_start_mana
        self.player.mana = Constants.player_start_mana
    
        self.goblin = Character.Character()
        self.goblin.name = 'Goblin'
        self.goblin.base_health = Constants.goblin_start_health
        self.goblin.health = Constants.goblin_start_health
        self.goblin.max_health = Constants.goblin_start_health
        self.goblin.classes['Warrior'] = Constants.goblin_start_warrior_level
        self.goblin.stamina = Constants.goblin_start_stamina
        self.goblin.base_stamina = Constants.goblin_start_stamina
        
        self.skills = {}
        
        # load the skills
        for skill_structure in Skills.skills:
            skill = Skill.Skill()
            skill.name = skill_structure[0]
            skill.positive_class = skill_structure[1]
            skill.positive_value = skill_structure[2]
            skill.negative_class = skill_structure[3]
            skill.negative_value = skill_structure[4]
            skill.required_class = skill_structure[5]
            skill.required_value = skill_structure[6]
            skill.target_type = skill_structure[7]
            skill.effect_type = skill_structure[8]
            skill.effect_amount = skill_structure[9]
            skill.effect_duration = skill_structure[10]
            skill.power_type = skill_structure[11]
            skill.power_cost =skill_structure[12]
            self.skills[skill_structure[0]] = skill

        # legal goblin attacks
        self.goblin_attacks = [self.skills['Stab'], 
                               self.skills['Cut'],
                               self.skills['Strike'],
                               self.skills['Slash'], 
                               self.skills['Thrust'],
                               self.skills['Cleave']]

        # seed the RNG
        random.seed()
    
    def apply_skill(self, skill, target):
        if skill.effect_type == 'Damage':
            target.damage(skill.effect_amount)
        elif skill.effect_type == 'Heal':
            target.heal(skill.effect_amount)
        elif skill.effect_type == 'Health Buff':
            buff = Buff.Buff()
            buff.effect_type = 'Health'
            buff.effect_amount = skill.effect_amount
            buff.timer = skill.effect_duration
            target.buff(buff)
        elif skill.effect_type == 'Defense Buff':
            buff = Buff.Buff()
            buff.effect_type = 'Defense'
            buff.effect_amount = skill.effect_amount
            buff.timer = skill.effect_duration
            target.buff(buff)
    
    def do_attack(self, attacker, target, skill):
        if attacker.classes[skill.required_class] >= skill.required_value:
            self.deduct_power_cost(attacker, skill)
            
            # base chance to hit is 50% + class level of skill
            hit_chance = Constants.base_chance_to_hit + (attacker.classes[skill.required_class] / Constants.skill_levels_to_increase_chance_to_hit)
            
            if hit_chance > Constants.max_chance_to_hit:
                hit_chance = Constants.max_chance_to_hit
            
            if random.random() * 100 <= hit_chance:
                print attacker.name + ' hits ' + target.name + ' with ' + skill.name + ' for ' + str(skill.effect_amount) + ' damage.'
                target.damage(skill.effect_amount)
            else:
                print attacker.name + ' misses.'
        else:
            print "Can't use skill " + skill.name + " because level is too low."

    def do_self_skill(self, character, skill):
        print character.name + " uses the " + skill.name + " skill."
        self.deduct_power_cost(character, skill)
        self.apply_skill(skill, character)

    def deduct_power_cost(self, target, skill):
        # deduct the power cost
        if skill.power_type == 'Stamina':
            target.tire(skill.power_cost)
        elif skill.power_type == 'Mana':
            target.drain(skill.power_cost)
        elif skill.power_type == 'Health':
            target.damage(skill.power_cost)

    def tick(self):
        print str(self.player)
        print str(self.goblin)

        # call tick on each character
        self.player.tick()
        self.goblin.tick()
        
        # if the player is alive make the player attack
        if self.player.is_alive():
            player_attack = None
            checks = 0
            
            while player_attack == None and checks < 50:
                checks = checks + 1
                
                # grab a random player attack
                player_attack = self.skills.values()[int(random.random() * len(self.skills))]
                
                if (player_attack.power_type == 'Health' and \
                    player_attack.power_cost > self.player.health) or \
                   (player_attack.power_type == 'Stamina' and \
                    player_attack.power_cost > self.player.stamina) or \
                   (player_attack.power_type == 'Mana' and \
                    player_attack.power_cost > self.player.mana) or \
                   (player_attack.required_class != None and \
                    self.player.classes[player_attack.required_class] < player_attack.required_value):
                    player_attack = None
            
            if player_attack != None:
                self.player.adjust_trend(player_attack)
                if player_attack.target_type == 'Self':
                    # apply the skill
                    self.do_self_skill(self.player, player_attack)
                else:
                    self.do_attack(self.player, self.goblin, player_attack)
            else:
                print self.player.name + ' looks too tired to attack.'
        
        # if the goblin is alive make the goblin attack
        if self.goblin.is_alive():
            gobby_attack = None
            checks = 0
            
            while gobby_attack == None and checks < 20:
                checks = checks + 1
                
                # grab a random goblin attack
                gobby_attack = self.goblin_attacks[int(random.random() * len(self.goblin_attacks))]
                
                if gobby_attack.power_cost > self.goblin.stamina:
                    gobby_attack = None
            
            if gobby_attack != None:
                self.goblin.adjust_trend(gobby_attack)
                self.do_attack(self.goblin, self.player, gobby_attack)
            else:
                print self.goblin.name + ' looks too tired to attack.'

        print str(self.player)
        print str(self.goblin)
        
        if self.goblin.is_dead() and self.player.is_alive():
            print self.player.name + " won."
            return
        elif self.goblin.is_alive() and self.player.is_dead():
            print self.goblin.name + " won."
            return
            
    
def main():
    sim = Sim()
    combat_round = 0
    
    # ticks
    while sim.goblin.is_alive() and sim.player.is_alive():
        combat_round = combat_round + 1
    
        print '========= ROUND ' + str(combat_round) + ' ==========='
        sim.tick()

if __name__ == "__main__":
    main()    